<?php

error_reporting(1);
ini_set('display_errors', '0');

//error_reporting(E_ALL);
//ini_set('display_errors', '1');

if ( ! defined('EXT')) exit('Invalid file request');

//
// In case exif is not enabled for this PHP installation
//
if ( ! function_exists( 'exif_imagetype' ) ) {
	function exif_imagetype ( $filename ) {
	  if ( ( list($width, $height, $type, $attr) = getimagesize( $filename ) ) !== false ) {
		  return $type;
	  }
	return false;
	}
}
//

/**
 * File Field Class
 *
 * @package   FieldFrame
 * @author    Fred Boyle @ nGen Works
 */
class Ngen_file_field extends Fieldframe_Fieldtype {

	/**
	 * Fieldtype Info
	 * @var array
	 */
	var $info = array(
		'name'     => 'nGen File Field',
		'version'  => '1.0',
		'desc'     => 'Provides a file fieldtype (extended by Denik)',
		'docs_url' => 'http://www.ngenworks.com/software/ee/',
		'versions_xml_url' => 'http://ngenworks.com/software/version-check/versions.xml'
	);
	
	var $hooks = array(
		// Field Manager
		'show_full_control_panel_end',
		'weblog_standalone_insert_entry'
	);

	var $default_field_settings = array(
		'options' => '',
		'custom_path' => ''
	);

	var $default_cell_settings = array(
		'options' => '',
		'custom_path' => ''
	);
	
	//
	var $postpone_saves = FALSE;
	
	var $default_site_settings = array(
		'quality_setting' => 'y'
	);
	
	var $upload_prefs = array();
	
	/**
	 * class constructor
	 */
	function __construct()
	{
		global $FF, $PREFS;
		// Set db_prefix
		$this->db_prefix = $PREFS->ini('db_prefix');
	}
	
	/**
	 * Display Site Settings
	 */
	function display_site_settings()
	{
		global $DB, $PREFS, $DSP;

		$SD = new Fieldframe_SettingsDisplay();

		$r = $SD->block()
		   . $SD->row(array(
		                  $SD->label('quality_setting_label', 'quality_setting_desc'),
		                  $SD->select('quality_setting', $this->site_settings['quality_setting'], array('y' => 'yes', 'n' => 'no'))
		              ))
		   . $SD->block_c();

		return $r;
	}

	/**
	 * Display - Show Full Control Panel - End
	 *
	 * - Rewrite CP's HTML
	 * - Find/Replace stuff, etc.
	 *
	 * @param  string  $end  The content of the admin page to be outputted
	 * @return string  The modified $out
	 * @see    http://expressionengine.com/developers/extension_hooks/show_full_control_panel_end/
	 */
	function show_full_control_panel_end($out) {
		global $SESS, $DSP;
		@session_start();
	
		$out = $this->get_last_call($out);
		
		//
		// Display any errors		
		if( !empty($_SESSION['ngen']['ngen-file-errors']) ) {
			$error_output = "<div class='ngen-file-errors'><ul>\n";
			foreach($_SESSION['ngen']['ngen-file-errors'] as $key => $error) {
				$error_output .= "<li";
				
				end($_SESSION['ngen']['ngen-file-errors']);
				
				if( $key == key($_SESSION['ngen']['ngen-file-errors']) ) {
					$error_output .= " class='last'";
				}
				$error_output .= ">" . $error . "</li>\n";
			}
			$error_output .= "</div>\n";
			
			if( preg_match("/\<div id=('|\")?contentNB('|\")?\>/", $out, $error_matches) ) {
				$out = str_replace($error_matches[0], $error_matches[0] . $error_output, $out);
			}
		}
		//
		
		//
		// Display any messages	
		if( !empty($_SESSION['ngen']['ngen-file-messages']) ) {
			$msg_output = "<div class='ngen-file-messages'><ul>\n";
			foreach($_SESSION['ngen']['ngen-file-messages'] as $error) {
				$msg_output .= "<li>" . $error . "</li>\n";
			}
			$msg_output .= "</div>\n";
			
			if( preg_match("/\<div id=('|\")?contentNB('|\")?\>/", $out, $msg_matches) ) {
				$out = str_replace($msg_matches[0], $msg_matches[0] . $msg_output, $out);
			}
		}
		//
								
		//
		// Make Publish Table multipart/form-data
		if(preg_match("/name=.entryform./", $out, $matches))
		{
			// Check to see if we already have the multipart stuff in the tag, add it if we don't
			if(! preg_match("/enctype=\"multipart\/form-data\"/", $matches[0]) ) {
				$out = str_replace($matches[0], $matches[0] . " enctype=\"multipart/form-data\"", $out);
			}
		}
		//
		
		if (isset($_SESSION) AND isset($_SESSION['ngen']))
		{
			if (isset($_SESSION['ngen']['ngen-file-errors'])) unset($_SESSION['ngen']['ngen-file-errors']);
			if (isset($_SESSION['ngen']['ngen-file-messages'])) unset($_SESSION['ngen']['ngen-file-messages']);
		}
		
		
		//exit('dead');

		return $out;
	}
	//
	//
	//
	
	/**
	 * Display Field Settings
	 * 
	 * @param  array  $field_settings  The field's settings
	 * @return array  Settings HTML (cell1, cell2, rows)
	 */
	function display_field_settings($field_settings)
	{
		global $DSP, $LANG;
		
		// Existing file show/hide option
		$hide_existing_selected = '';
		if(isset($field_settings['hide_existing']) && $field_settings['hide_existing'] == 'y') { $hide_existing_selected = 1; }
		       
		// Upload location select 
		$cell2 = $DSP->qdiv('defaultBold', $LANG->line('file_options_label'))
		       . $this->_select_upload_locations($field_settings['options'])
					 . $DSP->qdiv('defaultBold', $LANG->line('file_hide_existing_label'))
					 . $DSP->input_checkbox('hide_existing', 'y', $hide_existing_selected)
					 
					 . $DSP->qdiv('defaultBold', $LANG->line('file_custom_path_label'))
					 . $DSP->input_text('custom_path', $field_settings['custom_path'],null, '255')
		 			 . $DSP->qdiv('default', $LANG->line('file_custom_path_variables'));
		 			 
		$cell2 .= $DSP->qdiv('box',	$DSP->qdiv('defaultBold', $LANG->line('file_image_resize'))
					 . $DSP->input_text('image_w', isset($field_settings['image_w'])?$field_settings['image_w']:0,null, '25', 'input', "50px") . ' X '
					 . $DSP->input_text('image_h', isset($field_settings['image_h'])?$field_settings['image_h']:0,null, '25', 'input', "50px")
					 . $DSP->input_checkbox('image_resize_small', 'y', (isset($field_settings['image_resize_small'])&&$field_settings['image_resize_small']=='y')?'y':'n' ). " " . $LANG->line('file_image_resize_small')
		 			 . $DSP->qdiv('default', $LANG->line('file_image_resize_about')) );
					 
		$cell2 = $DSP->qdiv('rel_block', $cell2);

		return array('cell2' => $cell2);
	}
	//

	/**
	 * Display Cell Settings
	 * 
	 * @param  array  $cell_settings  The cell's settings
	 * @return string  Settings HTML
	 */
	function display_cell_settings($cell_settings)
	{
		global $DSP, $LANG;
		
		// Existing file show/hide option
		$hide_existing_selected = '';
		if(isset($cell_settings['hide_existing']) && $cell_settings['hide_existing'] == 'y') { $hide_existing_selected = 1; }
		
		$r = '<label class="itemWrapper">'
		   . $DSP->qdiv('defaultBold', $LANG->line('file_options_label'))
		   . $this->_select_upload_locations($cell_settings['options'])
		   . '</label>'
			 . '<label class="itemWrapper">'
 			 . $DSP->qdiv('defaultBold', $LANG->line('file_hide_existing_label'))
			 . $DSP->input_checkbox('hide_existing', 'y', $hide_existing_selected)
			 . '</label>';
			 
		$r .= '<label class="itemWrapper">'
		   . $DSP->qdiv('defaultBold', $LANG->line('file_custom_path_label'))
		   . $DSP->input_text('custom_path', $cell_settings['custom_path'],null, '255')
		   . $DSP->qdiv('default', $LANG->line('file_custom_path_variables'))
		   . '</label>';
		   
		$r .= '<label class="itemWrapper">'
		   . $DSP->qdiv('box',	$DSP->qdiv('defaultBold', $LANG->line('file_image_resize'))
		   . $DSP->input_text('image_w', isset($cell_settings['image_w'])?$cell_settings['image_w']:0,null, '25', 'input', "50px") . ' X '
		   . $DSP->input_text('image_h', isset($cell_settings['image_h'])?$cell_settings['image_h']:0,null, '25', 'input', "50px")
		   . $DSP->input_checkbox('image_resize_small', 'y', (isset($cell_settings['image_resize_small'])&&$cell_settings['image_resize_small']=='y')?'y':'n' ). " " . $LANG->line('file_image_resize_small')
		   . $DSP->qdiv('default', $LANG->line('file_image_resize_about')) )
  		   . '</label>';

		return $r;
	}	
	//

	/**
	 * Display Field
	 * 
	 * @param  mixed  $field_name
	 * @param  mixed  $field_data
	 * @param  array  $field_settings  The field's settings
	 * @return string  Field HTML
	 */
	function display_field($field_name, $field_data, $field_settings)
	{
		global $IN, $LANG, $PREFS;
		
		$LANG->fetch_language_file('ngen_file_field');
	
		$this->include_css('styles/ngen_file_field.css');
		$this->include_js('scripts/jquery.livequery.js');
		$this->include_js('scripts/jquery.ngen_file_field.js');
		
		$del_field_name = $field_name . "[delete]";
		$existing_field_name = $field_name . "[existing]";
		
		//
		$this->field_settings = $field_settings;
		
		$hide_choose_existing = @$field_settings['hide_existing'];

		// Check if field_data is an array or not
		if( !is_array($field_data) ) {
			$file_name = $field_data;
			$custom_path = "";
		} else {
			$file_name = array_key_exists('file_name', $field_data) ? $field_data['file_name'] : '';
			$custom_path = array_key_exists('custom_path', $field_data) ? $field_data['custom_path'] : '';
		}

		//
		// Make sure we're not on the edit field screen
		// this helps us avoid issues when trying to retrieve upload settings and file lists
		//
		$this->edit_field = false;
		
		if($IN->GBL('M', 'GET') == 'blog_admin' AND $IN->GBL('P', 'GET') == 'edit_field') {
			$this->edit_field = true;
		}
		//
		
		//
		if(!$this->edit_field) {
			$this->_get_upload_prefs($field_settings['options']);
			
			$file_path = $this->upload_prefs['server_path'] . $custom_path . $file_name;
			$file_uri = $this->upload_prefs['server_uri'] . $custom_path . $file_name;
			$file_url = $this->upload_prefs['server_url'] . $custom_path . $file_name;
		}
		//
		
		if( $file_name == "" && $custom_path == "" )
		{
			$custom_path = $this->_parse_custom_path($field_settings['custom_path']);
			if( ! @is_dir($this->upload_prefs['server_path'] . $custom_path) ) $custom_path = "";
		}
			
		$file_field = "<div class='ngen-file-field-block'>";
 	
		if( isset($file_name) AND $file_name ) {
			
			$file_field .= "<div class='ngen-file-field-data'>";
			
			$file_field .= "<a href='$file_url' target='_blank' class='ngen-file-link'>";
			
			$file_kind_text = $LANG->line('file_kind');
			
			// Is file an image?
			if($this->_is_image($file_path)) {
				$file_kind_text = $LANG->line('image_kind');
			
				// if thumbnail doesn't exist, create it.
				$img_info = $this->_image_info($file_path);
				
				$thumbnail = $this->upload_prefs['server_url'] . $custom_path . $img_info['thumbnail'];
				
				// Legacy for existing files
				if(!file_exists($this->upload_prefs['server_path'] . $custom_path . $img_info['thumbnail'])) {
					$thumbnail = $this->upload_prefs['server_url'] . $custom_path . $this->_create_thumbnail($file_path);
				} 
				
				$file_field .= "<img src='$thumbnail' class='ngen-file-thumbnail' alt='" . $file_name . "' />\n";
			}
			
			$file_field .= $file_name . "</a>\n";
			$file_field .= "<div class='ngen-ff-delete'>\n";
			$file_field .= "<a href='#' class='ngen-file-delete-button'>" . str_replace('%{file_name}', $file_name, $LANG->line('delete_file_link')) . "</a>\n";
			
			// Confirm whether to change, delete, or cancel
			$file_field .= "<div class='ngen-ff-choice' style='display: none;'>\n";
			$file_field .= "<ul>\n";
			// Hide remove option
			if(!$hide_choose_existing) {
				$file_field .= "<li class='ngen-ff-choice-remove'><a href='#'>" . str_replace('%{file_kind}', $file_kind_text, $LANG->line('choice_remove')) . "</li>";
			}
			//
			$file_field .= "<li class='ngen-ff-choice-delete'><a href='#'>" . $LANG->line('choice_delete') . "</li>";
			$file_field .= "</ul>\n";
			$file_field .= "<div class='ngen-ff-choice-bottom'></div>\n";
			$file_field .= "<a href='#' class='ngen-ff-choice-cancel'>Cancel</a>\n";
			$file_field .= "</div> <!-- close .ngen-ff-choice -->\n";
			//
			
			$file_field .= "</div> <!-- close .ngen-ff-delete -->\n";
			
			$file_field .= "</div>\n";
			
			$file_field .= "<div class='ngen-file-field-new' style='display: none;'>\n";
			$file_field .= "<input type='file' name='$field_name' class='ngen-file-input' />\n";
			$file_field .= "<input type='hidden' name='" . $field_name . "[custom_path]' value='" . $custom_path  . "' />\n";
			$file_field .= "<input type='hidden' name='" . $field_name . "[file_name]' value='" . $file_name  . "' />\n";
			$file_field .= "<input type='hidden' name='$del_field_name' />\n";
			
			// Existing file select
			if(!$hide_choose_existing) {
				$file_field .= "<div class='ngen-file-existing' style='display: none;'>\n";
				$file_field .= $this->_get_existing_select($existing_field_name, $custom_path);
				$file_field .= "</div>\n";
			
				$file_field .= "<div class='ngen-file-choose-existing'>" . $LANG->line('use_existing') . "</div>\n";
			}
			//
			
			$file_field .= "</div>\n";
			
		} else {
			// Empty field
			$file_field .= "<input type='file' name='$field_name' class='ngen-file-input' />\n";
			$file_field .= "<input type='hidden' name='" . $field_name . "[file_name]' />\n";
			
			// Existing file select
			if(!$hide_choose_existing) {
				$file_field .= "<div class='ngen-file-existing' style='display: none;'>\n";
				$file_field .= $this->_get_existing_select($existing_field_name, $custom_path);
				$file_field .= "</div>\n";
			
				$file_field .= "<div class='ngen-file-choose-existing'>" . $LANG->line('use_existing') . "</div>\n";
			}
			//
		}
		
		$file_field .= "</div>";
		
		//$js = 'nGenFile.lang.confirmDeleteFile = "'.$LANG->line('confirm_delete_file').'";';
		//$js .= 'nGenFile.lang.confirmRemoveFile = "'.$LANG->line('confirm_remove_file').'";';
		
		preg_match("~(.*?)(\[.+\]\[.+\])?$~", $field_name, $fn_matches);
		$field_name_js = $fn_matches[1];
		
		$js = '';
		$js .= 'nGenFile.lang.use_existing = "'.$LANG->line('use_existing').'";';
		$js .= 'nGenFile.lang.use_existing_cancel = "'.$LANG->line('use_existing_cancel').'";';
		$js .= 'nGenFile.lang.uploading = "'.$LANG->line('uploading').'";';
		$js .= 'nGenFile.thumbpaths["' . $field_name_js . '"] = "' . @$this->upload_prefs['server_url'] . '";';
		
		$this->insert_js($js);
		
		return $file_field;
	}
	//
	
	/**
	 * Display Cell
	 * 
	 * @param  mixed  $cell_name
	 * @param  array  $cell_data
	 * @param  array  $cell_settings  The cell's settings
	 * @return string  Cell HTML
	 */
	function display_cell($cell_name, $cell_data, $cell_settings)
	{
		return $this->display_field($cell_name, $cell_data, $cell_settings);
	}
	//
	
	//
	/**
	 * Save Field
	 * 
	 * @param  mixed  $field_data      The field's current value
	 * @param  array  $field_settings  The field's settings
	 * @return mixed	Modified or same $field_data
	 */
	function save_field($field_data, $field_settings)
	{
		global $FF, $IN, $SESS;
		
		@session_start();
		
		$field_name = $FF->field_name;

		// If delete field has value delete the file + thumbnail
		if(is_array($field_data) && isset($field_data['delete']) && $field_data['delete'] != '')
		{
			$this->_get_upload_prefs($field_settings['options']);
			if( !isset($field_data['custom_path']) ) $field_data['custom_path'] = "";
			
			if($this->_is_image($this->upload_prefs['server_path'] . $field_data['custom_path'] . $field_data['delete'])) {
				// delete all auto-creates thumbnail
				$file = $this->_pieces($field_data['delete']);
				$matching_files = glob($this->upload_prefs['server_path'] . $field_data['custom_path'] . "thumbs/" . $file['name'] . "_thumb_tmpl_" . "*." . $file['ext']);
				foreach( $matching_files as $file ) @unlink($file);
				
				$image_info = $this->_image_info($this->upload_prefs['server_path'] . $field_data['custom_path'] . $field_data['delete']);
				@unlink($this->upload_prefs['server_path'] . $field_data['custom_path'] . $image_info['thumbnail']);
			}
				
			@unlink($this->upload_prefs['server_path'] . $field_data['custom_path'] . $field_data['delete']);
			//$_SESSION['ngen']['ff-file-messages'][] = "File <em>" . $_POST[$field_name . "_delete"] . "</em> deleted.";

			$this->_remove_empty_folders($this->upload_prefs['server_path'], $field_data['custom_path']);

			// Remove delete variables to avoid saving issues
			unset($field_data['delete']);
		}

		// Create CUSTOM PATH 
		$field_settings['custom_path'] = isset($field_settings['custom_path'])? $this->_parse_custom_path($field_settings['custom_path']) : "";
		
		//if(empty($field_data['file_name']) && ( $_FILES[$field_name]['name'] != "" || $field_data['existing'] != "" ) ) {
			
		// update by Brandon Kelly for SAEF compatibility
		if(is_array($field_data) && empty($field_data['file_name']) && ( ( isset($_FILES[$field_name]) && $_FILES[$field_name]['name'] ) || (isset($field_data['existing']) && $field_data['existing']) ) ) {
		
			//unset($field_data['file_name']);
			//$existing_file = (isset($field_data['existing'])) ? $field_data['existing'] : NULL;
			//unset($field_data);
			unset($_POST[$field_name]);

			if(isset($field_data['existing'])) {
				// If using an existing file
				if( isset($field_data['custom_path']) ) {
					$file_name = array(
							'file_name' => $field_data['existing'],
							'custom_path' => $field_data['custom_path']
					);
				}
				else $file_name = $field_data['existing'];
			} else {
				// If uploading a new file
				$file_info['name'] = $_FILES[$field_name]['name'];
				$file_info['type'] = $_FILES[$field_name]['type'];
				$file_info['tmp_name'] = $_FILES[$field_name]['tmp_name'];
				$file_info['error'] = $_FILES[$field_name]['error'];
				$file_info['size'] = $_FILES[$field_name]['size'];
				
				$file_name = $this->upload_file($file_info, $field_settings);
			}
			
			$return = $file_name;
			
		} else {
			// Unset to make sure field is actually empty if no data is stored
			if(!$field_data['file_name']) {
				//unset($field_data['file_name']);
				unset($field_data);
				unset($_POST[$field_name]);
				
				$return = NULL;
				
			} else {
				if( is_array($field_data) && isset($field_data['custom_path']) && $field_data['custom_path']!='' )
				{
					$return = array(
						'file_name' => $field_data['file_name'],
						'custom_path' => $field_data['custom_path']
					);
				}
				else
				{
					$return = $field_data;
				}
			}
			
		}

		// IF SAEF - show errors 
		//
		if($IN->GBL('C', 'GET') == '') {
			return $this->_display_errors_SAEF();
		} else {
			return $return;
		}
		//
		
		//return $return;
	}
	//
	
	//
	/**
	 * Save Cell
	 * 
	 * @param  mixed  $cell_data      The cell's current value
	 * @param  array  $cell_settings  The cell's settings
	 * @return mixed	$cell_data
	 */
	function save_cell($cell_data, $cell_settings)
	{
		global $FF, $FFM, $IN, $SESS;
		
		@session_start();
			
		$field_name = $FF->field_name;
		$row_count = $FFM->row_count;
		$col_id = $FFM->col_id;
		
		//print_r($cell_data);
		//print_r($_FILES);

		// If delete field has value delete the file + thumbnail
		if (isset($cell_data['delete']))
		{
			if($cell_data['delete'])
			{
				$this->_get_upload_prefs($cell_settings['options']);
				$file_info = pathinfo($cell_data['delete']);
				if( !isset($cell_data['custom_path']) ) $cell_data['custom_path'] = "";
			
				if($this->_is_image($this->upload_prefs['server_path'] . $cell_data['custom_path'] . $cell_data['delete'])) {
					// delete all auto-creates thumbnail
					$file = $this->_pieces($cell_data['delete']);
					$matching_files = glob($this->upload_prefs['server_path'] . $cell_data['custom_path'] . "thumbs/" . $file['name'] . "_thumb_tmpl_" . "*." . $file['ext']);
					foreach( $matching_files as $file ) @unlink($file);
										
					$image_info = $this->_image_info($this->upload_prefs['server_path'] . $cell_data['custom_path'] . $cell_data['delete']);
					@unlink($this->upload_prefs['server_path'] . $cell_data['custom_path'] . $image_info['thumbnail']);
				}
				
				@unlink($this->upload_prefs['server_path'] . $cell_data['custom_path'] . $cell_data['delete']);
				//$_SESSION['ngen']['ff-file-messages'][] = "File <em>" . $_POST[$field_name . "_delete"][$row_count][$col_id] . "</em> deleted.";
				$this->_remove_empty_folders($this->upload_prefs['server_path'], $cell_data['custom_path']);
			}

			// Remove delete variables to avoid saving issues
			unset($cell_data['delete']);
		}
	
		// Create CUSTOM PATH 
		$cell_settings['custom_path'] = isset($cell_settings['custom_path'])? $this->_parse_custom_path($cell_settings['custom_path']) : "";

		if(
			$cell_data['file_name'] == '' &&
			(
				( isset($_FILES[$field_name]) && $_FILES[$field_name]['name'][$row_count][$col_id] != '' ) ||
				( $cell_data['existing'] != '' )
			)
		) {
		
			if($cell_data['existing']) {
				// If using existing file
				if( isset($cell_data['custom_path']) ) {
					$file_name = array(
							'file_name' => $cell_data['existing'],
							'custom_path' => $cell_data['custom_path']
					);
				}
				else $file_name = $cell_data['existing'];
				unset($cell_data['existing']);
			} else {
			
				// If uploading new file
				$file_info = array();
				$file_info['name'] = $_FILES[$field_name]['name'][$row_count][$col_id];
				$file_info['type'] = $_FILES[$field_name]['type'][$row_count][$col_id];
				$file_info['tmp_name'] = $_FILES[$field_name]['tmp_name'][$row_count][$col_id];
				$file_info['error'] = $_FILES[$field_name]['error'][$row_count][$col_id];
				$file_info['size'] = $_FILES[$field_name]['size'][$row_count][$col_id];
					
				$file_name = $this->upload_file($file_info, $cell_settings);
			}
			
			$return = $file_name;	
		
		} else {
		
			// Unset to make sure cell is actually empty if no data is stored
			if(!$cell_data['file_name']) {
				//unset($cell_data['file_name']);
				unset($_POST[$field_name . "_" . $row_count . "_" . $col_id]);
				
				$return = NULL;
			} else {
				if( is_array($cell_data) && isset($cell_data['custom_path']) && $cell_data['custom_path']!='' )
				{
					$return = array(
						'file_name' => $cell_data['file_name'],
						'custom_path' => $cell_data['custom_path']
					);
				}
				else
				{
					$return = $cell_data;
				}
			}
		}
		//
		
		// IF SAEF - show errors 
		//
		if($IN->GBL('C', 'GET') == '') {
			return $this->_display_errors_SAEF();
		} else {
			return $return;
		}
		//
		
		//return $return;
	}
	//
	
	//
	/**
	 * Upload File
	 * 
	 * @param  array  $upload_info    array built from instance's $_FILES data
	 * @param  array  $settings  			The cell/field's settings
	 * @return mixed	$file_name 			or false if an error
	 */
	function upload_file($upload_info, $settings) {
		global $LANG, $SESS, $IN, $DSP, $FNS, $EE;
		@session_start();
		
		$LANG->fetch_language_file('ngen_file_field');
		
		// Build File Pieces
		$file_name = $upload_info['name'];
		$file_type = $upload_info['type'];
		$file_tmp_name = $upload_info['tmp_name'];
		$file_error = $upload_info['error'];
		$file_size = $upload_info['size'];
				
		if($file_name) {
				
			// Clean && create url_name for file
			$file_name = $this->url_name($file_name);
			$file = $this->_pieces($file_name);
			//
			
			//
			$this->_get_upload_prefs($settings['options']);
			
			$upload_path = $this->upload_prefs['server_path'];
			$max_file_size = $this->upload_prefs['max_file_size'];
			$allowed_types = $this->upload_prefs['allowed_types'];
			$max_img_width = $this->upload_prefs['max_width'];
			$max_img_height = $this->upload_prefs['max_height'];
			
			// Are any of the file sizes too big?
			if($max_file_size != '')
			{
				if($file_size > $max_file_size)
				{
					$file_size_error = str_replace(array('%{file_name}', '%{max_size}'), array($file_name, $this->_size_readable($max_file_size, 'GB')), $LANG->line('error_file_size'));
				
					$_SESSION['ngen']['ngen-file-errors'][] = $file_size_error;
					
					$this->_error_message();
					
					return false;
				}
			}
			//
			
			// Is file of valid type? Applies only if limited to image type
			$is_image = exif_imagetype($file_tmp_name);
			
			if($allowed_types == 'img' && $is_image == false) {
				$_SESSION['ngen']['ngen-file-errors'][] = str_replace('%{file_name}', $file_name, $LANG->line('error_file_not_image'));
				
				$this->_error_message();
				
				unlink($file_tmp_name);
				return false;
			}
			//
			
			// If file is an image, are it's dimensions too large?
			list($img_width, $img_height) = getimagesize($file_tmp_name);
			
			if( $is_image && (($max_img_width > 0 && $img_width > $max_img_width) || ($max_img_height > 0 && $img_height > $max_img_height)) ) {
				$_SESSION['ngen']['ngen-file-errors'][] = str_replace(array('%{file_name}', '%{img_width}', '%{img_height}', '%{max_width}', '%{max_height}'), array($file_name, $img_width, $img_height, $max_img_width, $max_img_height), $LANG->line('error_image_dimensions'));
				
				$this->_error_message();
				
				unlink($file_tmp_name);
				return false;
			}
			//

			//
			// Check if file exists already, if it does add an increment to the file name
			if( file_exists($upload_path . $settings['custom_path'] . $file_name) ) {
				$matching_files = glob($upload_path . $settings['custom_path'] . $file['name'] . "*." . $file['ext']);
	
				// Find highest number, add 1 and set new $file_name
				natsort($matching_files);
				preg_match("/" . $file['name'] . "_(\d+)\." . substr($file['ext'], 1) . "/", basename(end($matching_files)), $matches);
				
				if( isset($matches[1]) && $matches[1] ) {
					$increment = "_" . ($matches[1] + 1);
				} else {
					$increment = "_1";
				}
				
				$file_name = $file['name'] . $increment . "." . $file['ext'];
				//
			}
			//
			
			/** create path structur {by Denik} */
			if( $settings['custom_path'] != '' )
			{
				$path = $upload_path;
				$exp_path = explode('/', $settings['custom_path']);
				foreach( $exp_path as $dir )
				{
					$path .= $dir.'/';
					if( !is_dir($path) ) mkdir($path, 0777);
					@chmod($path, 0777);
				}
			}
			
			// is image && resize it of config ?
			$this->_image_resize($file_tmp_name, $file_tmp_name, $settings['image_w'], $settings['image_h'], $settings['image_resize_small']);
			
			// Do the Upload
			if(@move_uploaded_file($file_tmp_name, $upload_path . $settings['custom_path'] . $file_name) === FALSE)
			{
				$_SESSION['ngen']['ngen-file-errors'][] = str_replace('%{file_name}', $file_name, $LANG->line('error_file_upload'));
				
				$this->_error_message();
				
				return false;
			} else {
				@chmod($upload_path . $settings['custom_path'] . $file_name, 0777);
				if( $this->_is_image($upload_path . $settings['custom_path'] . $file_name) ) {
					$this->_create_thumbnail($upload_path . $settings['custom_path'] . $file_name);
				}
				//$_SESSION['ngen']['ff-file-messages'][] = "File <em>$file_name</em> was successfully uploaded!";
			}
			//

			return array('file_name'=>$file_name,'custom_path'=>$settings['custom_path']);
		}
		
		return NULL;
	}
	//
	
	//
	/**
	 * Display Tag
	 *
	 * @param  array   $params          Name/value pairs from the opening tag
	 * @param  string  $tagdata         Chunk of tagdata between field tag pairs
	 * @param  string  $field_data      Currently saved field value
	 * @param  array   $field_settings  The field's settings
	 * @return string  relationship references
	 */
	function display_tag($params, $tagdata, $field_data, $field_settings)
	{
		global $TMPL;
	
		$r = '';
		
		if($field_settings['options'] && $field_data) {
			
			// Added by Fred Boyle - 2009.04.21
			// Check if field_data is an array or not
			if( !is_array($field_data) ) {
				$file_name = $field_data;
			} else {
				$file_name = ( isset($field_data['file_name']) ) ? $field_data['file_name'] : '';
				$custom_path = ( isset($field_data['custom_path']) ) ? $field_data['custom_path'] : '';
			}
			
			$this->_get_upload_prefs($field_settings['options']);
			
			if(isset($params['full_url']) && strtolower($params['full_url']) == 'yes')
				$file_url = $this->upload_prefs['server_url'] . $custom_path;
			else $file_url = $this->upload_prefs['server_uri'] . $custom_path;
			
			$file_path = $this->upload_prefs['server_path'] . $custom_path;
			
			// EXT & NAME
			$file = $this->_pieces($file_name);
			$file['name'];
			$file['ext'];
			
			$is_image = $this->_is_image($file_path.$file_name);
			
			// If image resize
			if( ( isset($params['width']) || isset($params['height']) ) && $is_image )
			{
				$w = isset($params['width'])?intval($params['width']):0;
				$h = isset($params['height'])?intval($params['height']):0;
				$resize_small = (isset($params['resize_small'])&&$params['resize_small']=='n') ? 'n' : 'y';
				$hor_align = isset($params['hor_align'])?$params['hor_align']:'C';
				$vrt_align = isset($params['vrt_align'])?$params['vrt_align']:'C';
				
				$thumb_name = $file['name'] . "_thumb_tmpl_{$hor_align}{$vrt_align}-{$resize_small}_{$w}x{$h}." . $file['ext'];
				
				if( ! file_exists($file_path . "/thumbs/" . $thumb_name) && file_exists($file_path.$file_name) )
				{
			    	if( !is_dir($file_path . "/thumbs/") ) mkdir($file_path . "/thumbs/", 0777);
					@chmod($file_path . "/thumbs/", 0777);
					$this->_image_resize($file_path.$file_name, "$file_path/thumbs/$thumb_name", $w, $h, $resize_small, $hor_align, $vrt_align);
				}
				$file_path .= "thumbs/";
				$file_url .= "thumbs/";
				$file_name = $thumb_name;
			}

			if( !empty($file_name) ) {
				$r = $file_url . $file_name; // default
			}
			
			// If show param is set
			if( isset($params['show']) )
			{
				$params['show'] = strtolower($params['show']);
				if( $params['show'] == 'filename' ) { 	// if only show file name
					$r = $file_name;
				} elseif( $params['show'] == 'ext' ) { 	// if only show file ext
					$r = $file['ext'];
				} elseif( $params['show'] == 'url' ) { // if only show file url
					$r = $file_url;
				} elseif( $params['show'] == 'path' ) { // if only show file path
					$r = $file_path;
				} elseif( $params['show'] == 'size' ) { // if only show file size
					$r = file_exists($file_path.$file_name)? filesize($file_path.$file_name) : 0;
				} elseif( $params['show'] == 'is_image' ) { // is image?
					$r = $is_image? 'y' : 'n';
				}
			}
		}
		
		return trim($r);
	}
	//
	
	/**
	 * Split a file to name/suffix
	 *
	 * @access private
	 * @param string, string
	 * @return string
	 */
	function _pieces($file_name)
	{
		/*$base = preg_replace('/^(.*)\.[^.]+$/', '\\1', $file_name);
		$ext = preg_replace('/^.*(\.[^.]+$)/', '\\1', $file_name);
		if( $ext == $file_name ) $ext = "";*/
		$base = $file_name;
		$exp_name = explode('.', $file_name);
		$ext = end($exp_name);
		if( count($exp_name)>1 )
		{
			array_pop($exp_name);
			$base = end($exp_name);
		}
		else $ext = "";
		
		return array('name'=>$base, 'ext'=>$ext);
	}
	//
	
	//
	// Builds the select for settings
	// returns HTML for select
	//
	function _select_upload_locations($current_option)
	{
		global $DB, $PREFS;
		
		// Add custom drop down for file location
		$block = "<div class='itemWrapper'><select name=\"options\"><option value=\"\"></option>";
		
		$dls = $DB->query("SELECT id, name FROM " . $this->db_prefix . "_upload_prefs WHERE site_id = " . $PREFS->ini('site_id') . " ORDER BY name ASC");
		foreach($dls->result as $dl)
		{
			$selected = ($dl['id'] == $current_option) ? " selected=\"true\"" : "";
			$block .= "<option value=\"{$dl['id']}\"$selected>{$dl['name']}</option>";
		}
		
		$block .= "</select></div></div>";
		
		return $block;
	}
	//
	
	//
	// Retrieves the upload preferences for an upload location
	// returns upload_prefs array
	//
	function _get_upload_prefs($u_id) {
		
		if( !isset($this->upload_prefs['loc_id']) || $this->upload_prefs['loc_id'] != $u_id ) {
			global $DB, $FNS, $PREFS;
			
			$query = $DB->query("SELECT * FROM " . $this->db_prefix . "_upload_prefs WHERE id = $u_id");
			
			$this->upload_prefs['loc_id'] = $u_id;
			$this->upload_prefs['server_path'] = trim($query->row['server_path']); // trim it just to be safe
			
			// Is this a relative path?
			// - check if path starts with ..
			//
			if( substr($this->upload_prefs['server_path'], 0, 2) == '..' ) {
				// found relative path, turn it into a proper absolute one
				// Use the PATH constant since it points to the CP path
				//$this->upload_prefs['server_path'] = PATH . $this->upload_prefs['server_path'];
				$this->upload_prefs['server_path'] = $FNS->remove_double_slashes($_SERVER['DOCUMENT_ROOT'] . substr($this->upload_prefs['server_path'], 2));
			}
			
			//$this->upload_prefs['server_uri'] = parse_url($query->row['url'], PHP_URL_PATH); // req. PHP 5.1.2
			
			// for PHP 4/5+
			$url_bits = parse_url($query->row['url']);
			$this->upload_prefs['server_uri'] = $url_bits['path'];
			
			//$upload_prefs['server_url'] = $FNS->remove_double_slashes( $PREFS->ini('site_url') . $upload_prefs['server_uri'] );
			$this->upload_prefs['server_url'] = $query->row['url'];
			
			// If the server URL is relative instead of a full URL then build a complete URL w/ hostname etc.
			if( strtolower(substr($this->upload_prefs['server_url'], 0, 5)) != 'http:' ) {
				$this->upload_prefs['server_url'] = $FNS->remove_double_slashes( $PREFS->ini('site_url') . $this->upload_prefs['server_url'] );
			}
			
			$this->upload_prefs['allowed_types'] = $query->row['allowed_types'];
			$this->upload_prefs['max_file_size'] = $query->row['max_size'];
			$this->upload_prefs['max_width'] = isset($query->row['max_width'])?intval($query->row['max_width']):0;
			$this->upload_prefs['max_height'] = isset($query->row['max_height'])?intval($query->row['max_height']):0;
		}
			
		//return $upload_prefs;
	}
	//
	
	//
	// Build or retrieve existing file list select/drop-down
	// returns HTML
	//
	function _get_existing_select($field_name, $custom_path = '') {
		global $LANG, $SESS;
		
		//preg_match("~(.*?)\[.*~", $field_name, $field_matches);
		//$field_id = $field_matches[1];

		$loc_id = isset($this->upload_prefs['loc_id'])?$this->upload_prefs['loc_id']:0;
		
		// If the existing file drop down already exists in the session use it, otherwise generate it
		if( isset($_SESSION['ngen']['ngen-file-existing'][$loc_id]) ) {
			
			// Make sure select has the proper field name
			$existing_html = "<select name='$field_name'>\n";
			$existing_html .= $_SESSION['ngen']['ngen-file-existing'][$loc_id];
			//$existing_html .= "<!-- from cache -->\n";
			
		} else {
			
			$existing_html = "<option value=''>" . $LANG->line('option_choose_existing') . "</option>\n";
			
			// No list to fetch if editing custom field
			if(!$this->edit_field) {		
				$existing_html .= $this->_get_file_list($this->upload_prefs['server_path'], true, $custom_path);
			}
			
			$existing_html .= "</select>\n";
			
			$_SESSION['ngen']['ngen-file-existing'][$loc_id] = $existing_html;
			
			// Make sure select has the proper field name
			$existing_html = "<select name='$field_name'>\n" . $existing_html;
			//$existing_html .= "<!-- NOT from cache -->\n";
			
		}
		
		return $existing_html;
	}
	//
	
	//
	// Retrieve list of files in folder
	// option to return as array or options for a select field
	//
	function _get_file_list($path, $as_options = false, $custom_path = '') {
		global $LANG;
		
		$LANG->fetch_language_file('ngen_file_field');
		
		//echo "Path: $path<br/>\n";
	
		$output = '';
		$file_list = array();
		
		// if path doesn't exist, create it
		if(!file_exists($path)) {  
			$create_dir = mkdir($path, 0777);
			@chmod($path, 0777);
			
			// Possible error fix to be tested, testing for failure of directory creation 
			if(!$create_dir) {
			
				$error_line = str_replace(array('%{upload_path}', '%{upload_edit_link}'), array($path, BASE . "&C=admin&M=blog_admin&P=edit_upload_pref&id=" . $this->field_settings['options']), $LANG->line('error_file_path'));
			
				if(!in_array($error_line, $_SESSION['ngen']['ngen-file-errors'])) {
					$_SESSION['ngen']['ngen-file-errors'][] = $error_line;
				}
				return false;
			}
		}
		//
		
		// if folder does not exists - clear the custom path
		if( !(file_exists($path . $custom_path) && is_dir($path . $custom_path)) ) $custom_path = '';
		
		$dir = new DirectoryIterator($path . $custom_path);
		
		foreach($dir as $fileinfo) {
			if(!$fileinfo->isDir()) {
				$filename = $fileinfo->getFilename();
			
				$file_list[] = $filename;
				
				if( $this->_is_image($path . $custom_path . $filename) ) {
					$thumb = $this->_create_thumbnail($path . $custom_path . $filename);
				}
				
			}
		}
		
		natcasesort($file_list);
		
		foreach($file_list as $key => $file) {
		
			if( substr($file, 0, 1) == "." ) {
				continue;
			}
		
			if($as_options) {
				$output .= "<option custom_path='".(($custom_path!='')?$custom_path:'')."' value='$file'>$file</option>\n";
			} else {
				$output .= $file . "\n";
			}
		}
		
		return $output;
	}
	//
	
	//
	// Remove empty folders (custom folders)
	//
	function _remove_empty_folders($server_path, $custom_path = '')
	{
		if( $custom_path == '' ) return;
		
		if( substr($custom_path, -1, 1)=='/' ) $custom_path =  substr($custom_path, 0, -1); // del last slash
		$exp_path = explode('/', $custom_path);
		$exp_path[] = 'thumbs';
		
		while( count($exp_path)>0 )
		{
			$f = implode('/',$exp_path);
			if( @is_dir($server_path . $f) && $handle = @opendir($server_path . $f) )
			{
				$count = 0;
				while($file = @readdir($handle)) $count++;
				@closedir($handle);
				if( $count <= 2 ) @rmdir($server_path . $f);
			}
			unset( $exp_path[count($exp_path)-1] );
		}
	}
	//
	
	//
	// Parse custom path fields
	//
	function _parse_custom_path($custom_path)
	{
		GLOBAL $DB, $IN, $PREFS;

		if( $custom_path=='' ) return '';
		$custom_path = str_replace(' ', '', $custom_path);
		
		$entry = intval($IN->GBL('entry_id')) ? "AND wt.entry_id='".$IN->GBL('entry_id')."'" : "ORDER BY wt.entry_id DESC LIMIT 0,1";
		$entry = $DB->query("SELECT wt.entry_id,wt.year,wt.month,wt.day,w.weblog_id,w.blog_name FROM exp_weblog_titles AS wt LEFT JOIN exp_weblogs as w ON w.weblog_id=wt.weblog_id WHERE wt.site_id = '".$PREFS->ini('site_id')."' $entry"); 
		$entry = $entry->row;
		
		if( ! $IN->GBL('entry_id') ) // Next entry_id & others
		{
			$entry['entry_id'] = $DB->query("SHOW TABLE STATUS LIKE 'exp_weblog_titles'"); 
			$entry['entry_id'] = $entry['entry_id']->row['Auto_increment'];
			$entry['day'] = date("d");
			$entry['month'] = date("m");
			$entry['year'] = date("Y");
		}
		
		foreach( $entry as $key=>$val )
		{
			$custom_path = str_replace(LD.$key.RD, $val, $custom_path);
		}
		
		$custom_path = preg_replace("#[\\\\/]+#is", '/', $custom_path);
		if( substr($custom_path, -1, 1)!='/' ) $custom_path .= '/';		// add last slash
		if( substr($custom_path, 0, 1)=='/' ) $custom_path = substr($custom_path, 1);	// del first slash
		return $custom_path;
	}
	//
	
	//
	// Parse custom path fields
	//
	function url_name($name)
	{
		GLOBAL $PREFS;

		if( function_exists('iconv') )
		{
			$chaset = $PREFS->ini('charset')?$PREFS->ini('charset'):'utf-8';
			$name = iconv($chaset, 'cp1251', $name);
		}

		// EXT & NAME
		$exp_name = $this->_pieces($name);
		$name = $exp_name['name'];
		$ext = ".".$exp_name['ext'];
		
		$tmp = "";
		$name = preg_replace('/[ _]+/i', '_', $name);
		$name = preg_replace('/[-]+/i', '-', $name);
		for( $i=0; $i<strlen($name); $i++ )
		{
			$c = ord($name[$i]);
			
			if( $name[$i]=='_' || $name[$i]=='-' || $name[$i]=='.' ) {$tmp .= $name[$i]; continue;}
			if( $c >= 48 && $c <= 57 ) {$tmp .= $name[$i]; continue;}	// 0, 1, 2,...
			if( $c >= 65 && $c <= 90 ) {$tmp .= $name[$i]; continue;}	// A, B, C,...
			if( $c >= 97 && $c <= 122 ) {$tmp .= $name[$i]; continue;}	// a, b, c,...
			
			if( $c == '192' ) {$tmp .= 'a'; continue;}
			if( $c == '193' ) {$tmp .= 'b'; continue;}
			if( $c == '194' ) {$tmp .= 'v'; continue;}
			if( $c == '195' ) {$tmp .= 'g'; continue;}
			if( $c == '196' ) {$tmp .= 'd'; continue;}
			if( $c == '197' ) {$tmp .= 'e'; continue;}
			if( $c == '198' ) {$tmp .= 'zh'; continue;}
			if( $c == '199' ) {$tmp .= 'z'; continue;}
			if( $c == '200' ) {$tmp .= 'i'; continue;}
			if( $c == '201' ) {$tmp .= 'j'; continue;}
			if( $c == '202' ) {$tmp .= 'k'; continue;}
			if( $c == '203' ) {$tmp .= 'l'; continue;}
			if( $c == '204' ) {$tmp .= 'm'; continue;}
			if( $c == '205' ) {$tmp .= 'n'; continue;}
			if( $c == '206' ) {$tmp .= 'o'; continue;}
			if( $c == '207' ) {$tmp .= 'p'; continue;}
			if( $c == '208' ) {$tmp .= 'r'; continue;}
			if( $c == '209' ) {$tmp .= 's'; continue;}
			if( $c == '210' ) {$tmp .= 't'; continue;}
			if( $c == '211' ) {$tmp .= 'u'; continue;}
			if( $c == '212' ) {$tmp .= 'f'; continue;}
			if( $c == '213' ) {$tmp .= 'h'; continue;}
			if( $c == '214' ) {$tmp .= 'c'; continue;}
			if( $c == '215' ) {$tmp .= 'ch'; continue;}
			if( $c == '216' ) {$tmp .= 'sh'; continue;}
			if( $c == '217' ) {$tmp .= 'sch'; continue;}
			if( $c == '219' ) {$tmp .= 'y'; continue;}			
			if( $c == '221' ) {$tmp .= 'e'; continue;}
			if( $c == '222' ) {$tmp .= 'yu'; continue;}
			if( $c == '223' ) {$tmp .= 'ya'; continue;}
			
			if( $c == '224' ) {$tmp .= 'a'; continue;}
			if( $c == '225' ) {$tmp .= 'b'; continue;}
			if( $c == '226' ) {$tmp .= 'v'; continue;}
			if( $c == '227' ) {$tmp .= 'g'; continue;}
			if( $c == '228' ) {$tmp .= 'd'; continue;}
			if( $c == '229' ) {$tmp .= 'e'; continue;}
			if( $c == '230' ) {$tmp .= 'zh'; continue;}
			if( $c == '231' ) {$tmp .= 'z'; continue;}
			if( $c == '232' ) {$tmp .= 'i'; continue;}
			if( $c == '233' ) {$tmp .= 'j'; continue;}
			if( $c == '234' ) {$tmp .= 'k'; continue;}
			if( $c == '235' ) {$tmp .= 'l'; continue;}
			if( $c == '236' ) {$tmp .= 'm'; continue;}
			if( $c == '237' ) {$tmp .= 'n'; continue;}
			if( $c == '238' ) {$tmp .= 'o'; continue;}
			if( $c == '239' ) {$tmp .= 'p'; continue;}
			if( $c == '240' ) {$tmp .= 'r'; continue;}
			if( $c == '241' ) {$tmp .= 's'; continue;}
			if( $c == '242' ) {$tmp .= 't'; continue;}
			if( $c == '243' ) {$tmp .= 'u'; continue;}
			if( $c == '244' ) {$tmp .= 'f'; continue;}
			if( $c == '245' ) {$tmp .= 'h'; continue;}
			if( $c == '246' ) {$tmp .= 'c'; continue;}
			if( $c == '247' ) {$tmp .= 'ch'; continue;}
			if( $c == '248' ) {$tmp .= 'sh'; continue;}
			if( $c == '249' ) {$tmp .= 'sch'; continue;}
			if( $c == '251' ) {$tmp .= 'y'; continue;}
			if( $c == '253' ) {$tmp .= 'e'; continue;}
			if( $c == '254' ) {$tmp .= 'yu'; continue;}
			if( $c == '255' ) {$tmp .= 'ya'; continue;}
		}
		$name = $tmp;
		
		// New name is NULL ? Generate New!
		if( $name == '' )
		{
			$name = md5( time() );
		}

		return strtolower($name.$ext);
	}
	
	//
	// Returns array containing filename, width, height
	//
	function _image_info($file) {
		// legacy for MH File compatibility
		$file = trim($file);
	
		$data = getimagesize($file);
		
		$file_info = pathinfo($file);
		$filename_noext = basename($file_info['basename'], "." . $file_info['extension']);
		
		$info = array();
		$info['filename'] = basename($file);
		$info['thumbnail'] = "thumbs/" . $filename_noext . "_thumb.".$file_info['extension'];
		$info['width'] = $data[0];
		$info['height'] = $data[1];
		$info['width_height'] = $data[3];
		$info['image_type'] = $data[2];
		
		return $info;
	}
	//
	
	//
	// Checks if a file is an image
	//
	function _is_image($file) {
		global $FNS;
		
		$is_image = false;
		
		// Denik add, check of ext:
		$img_exts = array('jpg', 'jpeg', 'gif', 'bmp', 'png');
		$ext = strtolower( substr($file, strrpos($file,'.')+1) );
		if( ! in_array($ext, $img_exts) ) return $is_image;
		
		// legacy for MH File compatibility
		$file = trim($file);
		
		$file = $FNS->remove_double_slashes($file);
		
		//
		// Check to make sure file is at least 12bytes, otherwise fail as not image - causes issues otherwise
		//
		
		if(file_exists($file) && filesize($file) > 11) {
					
			switch( @exif_imagetype($file) ) {
				case IMAGETYPE_GIF:
		//		case IMAGETYPE_BMP:
				case IMAGETYPE_JPEG:
				case IMAGETYPE_PNG:
					$is_image = true;
					break;
			}
		
		}
		
		return $is_image;
	}
	//
	
	function _image_resize($file_src, $file_dst, $width = 0, $height = 0, $resize_small = 'n', $hor_align = 'C', $vrt_align = 'C')
	{
		if( !( $width || $height ) ) return true;
		
		$vals = $this->_image_info($file_src);
		if( $vals['width'] == 0 ) return false; // no image
		
		// Upload & Resize Original foto
		if( $width || $height )
		{
			$new_w = $width ? $width: ceil($vals['width']/($vals['height']/$height));
			$new_h = $height ? $height: ceil($vals['height']/($vals['width']/$width));
		}
		else
		{
			$new_w = $vals['width'];
			$new_h = $vals['height'];
		}
		
		// Crop ?
		if( $width && $height )
		{
			$newc_w = $width;
			$newc_h = ceil($vals['height']/($vals['width']/$new_w));
			
			if( $newc_h < $height )
			{
				$newc_h = $height;
				$newc_w = ceil($vals['width']/($vals['height']/$new_h));
			}
		}
		else
		{
			$newc_w = $new_w;
			$newc_h = $new_h;
		}
		
		// No resize small images ?
		if( $resize_small!='y' && $vals['width']<=$new_w && $vals['height']<=$new_h )
		{
			return true;
		}
		
		@ini_set("memory_limit","12M");
		@ini_set("memory_limit","16M");
		@ini_set("memory_limit","32M");
		@ini_set("memory_limit","64M");
		@ini_set("memory_limit","128M");
		@ini_set("memory_limit","256M");
		
		$blending = false;
		switch( $vals['image_type'] ) {
			case IMAGETYPE_GIF:
				$im = imagecreatefromgif($file_src);
				$blending = true;
				break;
				
			case IMAGETYPE_JPEG:
				$im = imagecreatefromjpeg($file_src);
				break;
			
			case IMAGETYPE_PNG:
				$im = imagecreatefrompng($file_src);
				$blending = false;
				break;
		}
		
	    // If setting set to high quality thumbnails
	    if( $this->site_settings['quality_setting'] == 'y' )
		{
		    $nm = imagecreatetruecolor($new_w, $new_h);
	 	} else {
		  	$nm = imagecreate($new_w, $new_h);
		}
	    //
		    
	    // If GIF/PNG preserve transparency
	    if( $vals['image_type'] == IMAGETYPE_GIF || $vals['image_type'] == IMAGETYPE_PNG ) {
		  	// find the existing transparent color if it exists
		  	$transparencyIndex = imagecolortransparent($im);
		  	
		  	if ($transparencyIndex >= 0) {
		  		$transparencyColor = imagecolorsforindex($im, $transparencyIndex);
				$transparencyIndex = imagecolorallocate($im, $transparencyColor['red'], $transparencyColor['green'], $transparencyColor['blue']);
		  	}
	    	
	    	// define a color as transparent  
	    	imagecolortransparent($nm, $transparencyIndex);
	    	// set the blending mode for thumbnail
	    	imagealphablending($nm, $blending);
	    	// set the flag to save alpha channel
	    	imagesavealpha($nm, true);
	    }
	    //
		
		$x_axis = 0;
		$y_axis = 0;
		
		/** -------------------------------------
		/**  Set the vertical position
		/** -------------------------------------*/

		switch ($vrt_align)
		{
			case 'T':
				break;
			case 'C':
			case 'M':
				$y_axis += ($new_h / 2) - ($newc_h / 2);
				break;
			case 'B':	$y_axis += $new_h - $newc_h;
				break;
		}
		
		/** -------------------------------------
		/**  Set the horizontal position
		/** -------------------------------------*/
	
		switch ($hor_align)
		{
			case 'L':
				break;	
			case 'C':
			case 'M':
				$x_axis += ($new_w / 2) - ($newc_w / 2);
				break;
			case 'R':	$x_axis += $new_w - $newc_w;
				break;
		}
		//echo "$x_axis, $y_axis";exit();
		imagecopyresampled($nm, $im, $x_axis, $y_axis, 0, 0, $newc_w, $newc_h, $vals['width'], $vals['height']);
		imagedestroy($im);
				
		$img_saved = false;
		switch( $vals['image_type'] ) {
			case IMAGETYPE_GIF:
				$img_saved = imagegif($nm, $file_dst);
				break;
				
			case IMAGETYPE_JPEG:
				$img_saved = imagejpeg($nm, $file_dst, 90);
				break;
			
			case IMAGETYPE_PNG:
				$img_saved = imagepng($nm, $file_dst, 0);
				break;
		}
		
		imagedestroy($nm);

		// Make sure image was saved
	 	if( !$img_saved ) {
	 		return false;
	 	}

		@chmod($file_dst, 0777);
		
		return true;
	}
	
	//
	// Creates a thumbnail and returns the relative path to it
	//
	function _create_thumbnail($file, $width = 50, $height = 50) {
		global $LANG;
		
		$LANG->fetch_language_file('ngen_file_field');
	
		$uri = '';
		
		// legacy for MH File compatibility
		$file = trim($file);
		
		$file_info = pathinfo($file);
		$filename_noext = basename($file_info['basename'], "." . $file_info['extension']);
		
		$server_path = $file_info['dirname'];
		$thumb_path = $server_path . "/thumbs/";
		$filename = $file_info['basename'];
		//$thumb_name = $filename_noext . "_thumb.jpg";
		$thumb_name = $filename_noext . "_thumb." . $file_info['extension'];
		
		// if file is newer than thumb recreate the thumb
		if( !file_exists($thumb_path . $thumb_name) || (filemtime($file) > filemtime($thumb_path . $thumb_name)) ) {
		
			$image_info = $this->_image_info($file);
			
			// If image is smaller than thumbnail size, don't resize/create thumb just copy
			if($image_info['width'] < $width && $image_info['height'] < $height) {
			//if(false == true) {
				
				copy($file, $thumb_path . $thumb_name);
				
			} else {
			
				// Attempt to increase memory as much as possible
				@ini_set("memory_limit","12M");
				@ini_set("memory_limit","16M");
				@ini_set("memory_limit","32M");
				@ini_set("memory_limit","64M");
				@ini_set("memory_limit","128M");
				@ini_set("memory_limit","256M");
				
			
				switch( $image_info['image_type'] ) {
					case IMAGETYPE_GIF:
						$im = imagecreatefromgif($file);
						$blending = true;
						break;
						
					case IMAGETYPE_JPEG:
						$im = imagecreatefromjpeg($file);
						break;
					
					case IMAGETYPE_PNG:
						$im = imagecreatefrompng($file);
						$blending = false;
						break;
				}
				
		    $width_old = $image_info['width'];
		    $height_old = $image_info['height'];
		    
		    // Make sure we don't distort image, crop if needed
				$int_width = 0;
				$int_height = 0;
				
				$adjusted_height = $height;
				$adjusted_width = $width;
				
				$wm = $width_old/$width;
				$hm = $height_old/$height;
				$h_height = $height/2;
				$w_height = $width/2;
					
				$ratio = $width/$height;
				$old_img_ratio = $width_old/$height_old;
					
				if ($old_img_ratio > $ratio) 
				{
					$adjusted_width = $width_old / $hm;
					$half_width = $adjusted_width / 2;
					$int_width = $half_width - $w_height;
				} 
				else if($old_img_ratio <= $ratio) 
				{
					$adjusted_height = $height_old / $wm;
					$half_height = $adjusted_height / 2;
					$int_height = $half_height - $h_height;
				}
		    //
		    
		    
		    // If setting set to high quality thumbnails
		    if( $this->site_settings['quality_setting'] == 'y' ) {
			    $nm = imagecreatetruecolor($width, $height);
			  } else {
			  	$nm = imagecreate($width, $height);
			  }
		    //
		    // Dirtier thumbnails but less memory usage?
		    //$nm = imagecreate($width, $height);
		    
		    // If GIF/PNG preserve transparency
		    if( $image_info['image_type'] == IMAGETYPE_GIF || $image_info['image_type'] == IMAGETYPE_PNG ) {		  	
			  	// find the existing transparent color if it exists
			  	$transparencyIndex = imagecolortransparent($im);
			  	
			  	if ($transparencyIndex >= 0) {
			  		$transparencyColor = imagecolorsforindex($im, $transparencyIndex);
			  	}
			  	
			  	$transparencyIndex = imagecolorallocate($new_image, $transparencyColor['red'], $transparencyColor['green'], $transparencyColor['blue']);
			  	
			  	// allocate a color for thumbnail  
		    	//$background = imagecolorallocate($nm, 0, 0, 0);  
		    	//imagefill($nm, 0, 0, $transparencyIndex);
		    	
		    	// define a color as transparent  
		    	imagecolortransparent($nm, $transparencyIndex);
		    	// set the blending mode for thumbnail
		    	imagealphablending($nm, $blending);
		    	// set the flag to save alpha channel
		    	imagesavealpha($nm, true);
		    }
		    //
		    
		    imagecopyresampled($nm, $im, 0, 0, 0, 0, $adjusted_width, $adjusted_height, $width_old, $height_old); 
		    
		    imagedestroy($im);
		      
		    if(!file_exists($thumb_path)) {  
					$create_dir = mkdir($thumb_path, 0777);
					@chmod($thumb_path, 0777);
					
					
					// Testing for failure of directory creation 
					if(!$create_dir) {
					
						$error_line = str_replace(array('%{upload_path}', '%{upload_edit_link}'), array($thumb_path, BASE . "&C=admin&M=blog_admin&P=edit_upload_pref&id=" . $this->field_settings['options']), $LANG->line('error_file_path'));
				
						if(!in_array($error_line, $_SESSION['ngen']['ngen-file-errors'])) {
							$_SESSION['ngen']['ngen-file-errors'][] = $error_line;
						}
						
						$this->_error_message();
						return false;
					}
					
				}
				
				$img_saved = false;
				switch( $image_info['image_type'] ) {
					case IMAGETYPE_GIF:
						$img_saved = imagegif($nm, $thumb_path . $thumb_name);
						break;
						
					case IMAGETYPE_JPEG:
						$img_saved = imagejpeg($nm, $thumb_path . $thumb_name, 100);
						break;
					
					case IMAGETYPE_PNG:
						$img_saved = imagepng($nm, $thumb_path . $thumb_name, 0);
						break;
				}
				
				// Make sure image was saved
			 	if( !$img_saved ) {
			 		return false;
			 	}
			 	
			 	imagedestroy($nm);
			}
			//
			
			chmod($thumb_path . $thumb_name, 0777);
			$uri = "thumbs/" . $thumb_name;

		}
		
		return $uri;
	}
	//
	
	/**
	* Modify any of the POST data for a stand alone entry insert
	*
	* @return	null
	* @since 	Version 1.0.0
	* @see		http://expressionengine.com/developers/extension_hooks/weblog_standalone_insert_entry/
	*/
	function weblog_standalone_insert_entry() {
		//echo "Uploading...";
		return $this->_display_errors_SAEF();
	}
	//
	
	//
	//
	//
	function _error_message() {
		global $IN, $FNS, $SESS, $OUT;
		
		// IF SAEF - show errors 
		//
		if($IN->GBL('C', 'GET') == '') {
			$saef = TRUE;
		} else {
			$saef = FALSE;
		}
		//
		
		//
		if($saef) {
			//
			$this->_display_errors_SAEF();
			
		} else {
			//
			$e_weblog_id = $IN->GBL('weblog_id');
			$e_entry_id = $IN->GBL('entry_id');
		
			$url  = BASE . '&C=edit&M=edit_entry&weblog_id=' . $e_weblog_id;
			$url .= '&entry_id=' . $e_entry_id;
		}
		
		$FNS->redirect($url);
	}
	//
	//
	//
	
	//
	// Inserts error messages in a SAEF
	//
	function _display_errors_SAEF() {
		global $SESS, $OUT;
	
		@session_start();
				
		//
		// Display any errors
		if( !empty($_SESSION['ngen']['ngen-file-errors']) ) {
		
			$error_array = array();
		
			foreach($_SESSION['ngen']['ngen-file-errors'] as $key => $error) {
				$error_array[] = $error;
			}
			
			//
			if (isset($_SESSION) AND isset($_SESSION['ngen']))
			{
				if (isset($_SESSION['ngen']['ngen-file-errors'])) unset($_SESSION['ngen']['ngen-file-errors']);
				if (isset($_SESSION['ngen']['ngen-file-messages'])) unset($_SESSION['ngen']['ngen-file-messages']);
			}
			//
			
			//return $error_output;
			return $OUT->show_user_error('general', $error_array);
		}
		//
		
	}
	//
	
	/**
	 * Return human readable sizes
	 *
	 * @author      Aidan Lister <aidan@php.net>
	 * @version     1.1.0
	 * @link        http://aidanlister.com/repos/v/function.size_readable.php
	 * @param       int    $size        Size
	 * @param       int    $unit        The maximum unit
	 * @param       int    $retstring   The return string format
	 * @param       int    $si          Whether to use SI prefixes
	 */
	function _size_readable($size, $unit = null, $retstring = null, $si = true)
	{
	    // Units
	    if ($si === true) {
	        $sizes = array('B', 'kB', 'MB', 'GB', 'TB', 'PB');
	        $mod   = 1000;
	    } else {
	        $sizes = array('B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB');
	        $mod   = 1024;
	    }
	    $ii = count($sizes) - 1;
	 
	    // Max unit
	    $unit = array_search((string) $unit, $sizes);
	    if ($unit === null || $unit === false) {
	        $unit = $ii;
	    }
	 
	    // Return string
	    if ($retstring === null) {
	        $retstring = '%01.2f %s';
	    }
	 
	    // Loop
	    $i = 0;
	    while ($unit != $i && $size >= 1024 && $i < $ii) {
	        $size /= $mod;
	        $i++;
	    }
	 
	    return sprintf($retstring, $size, $sizes[$i]);
	}
	//
}


/* End of file ft.ff_file_field.php */
/* Location: ./system/fieldtypes/ff_file_field/ft.ff_file_field.php */